## NicheFunctions


## ----------------------------------------------------------------
##            Functions to calculate niche overlaps
## ----------------------------------------------------------------


## Niche overlap for binomial data:

no.bin.fn <- function(sp,yy){
    counts.mat <- as.matrix(table(sp,yy))
    props.mat  <- counts.mat/apply(counts.mat,1,sum)
    out.mat <- matrix(1,ns,ns)
    for (ii in 1:(ns-1)) for (jj in (ii+1):ns){
        small.mat <- props.mat[c(ii,jj),]
        mins <- apply(small.mat,2,min)
        out.mat[ii,jj] <- sum(mins)
        out.mat[jj,ii] <- sum(mins)
    }
    out.mat
}


## Niche overlap for categorical data (same as for bin data):

no.cat.fn <- function(sp,yy){
   counts.mat <- as.matrix(table(sp,yy))
   props.mat  <- counts.mat/apply(counts.mat,1,sum)
   out.mat <- matrix(1,np.spp,ns)
   for (ii in 1:(ns-1)){
       for (jj in (ii+1):ns){
           small.mat <- props.mat[c(ii,jj),]
           mins <- apply(small.mat,2,min)
           out.mat[ii,jj] <- sum(mins)
           out.mat[jj,ii] <- sum(mins)
       }
   }
   out.mat
}

## Modified log function to avoid log(0):

log.fn <- function(xx){
    xx[xx < 0.000001] <- 0.000001
    log(xx)
}


## Niche overlap for count data:

no.count.fn <- function(these.ss,yy){
    ## For each species or site, use the observed counts to fit a
    ## Poisson mixture model.
    ## Try models up to a maximum of four groups.
    ## Evaluate probabilities up to a high number of counts, 
    ## e.g. up to 2*max(yy)+1.
    maxcol <- 2*max(yy)+1
    pois.probs.mat <- matrix(NA,ns,maxcol)
    rownames(pois.probs.mat) <- snames
    for (sp in 1:ns){
        ## print(paste("Species number",sp))
        x <- yy[these.ss==snames[sp]]
        ## print(x)
        x <<- x ## Needed in 'optim' later
        ## How many groups to try? Set a maximum of four.
        Gmax <- min(4,ceiling(0.5*length(unique(x))))
        probs.mat <- matrix(NA,Gmax,maxcol)
        AIC.vect <- rep(NA,Gmax)
        ## One group.
        lamhat  <- mean(x)
        max.ll  <- sum(x) * (log.fn(mean(x))-1)
        res.dev <- -2*max.ll
        npar    <- 1
        AIC.vect[1] <- res.dev + 2*npar
        probs.mat[1,] <- dpois(0:(maxcol-1),lamhat)
        ## More than one group:
        if (Gmax > 1){
            for (g in 2:Gmax){
                g <<- g
                alpha.start <- rep(0.5,g-1)
                pi.start <- rep(1/g,g-1)
                ## Allows for a new group at end, prob 1/g
                start.vect <- c(alpha.start,max(x),pi.start)
                ## Fit the current model:
                g.fit <-  optim(par=start.vect,
                                fn=pois.minusll.g,
                                method = "L-BFGS-B",
                                lower=c(rep(0.0001,(2*g-1))),
                                upper=c(rep(Inf,g),rep(0.9999,(g-1))))
                ## Save the results:
                max.ll     <- -g.fit$value
                res.dev    <- -2*max.ll
                npar       <- 2*g-1
                AIC.vect[g] <- res.dev + 2*npar
                alpha.out  <- g.fit$par[1:(g-1)]
                lam.g      <- g.fit$par[g]
                pi.out     <- g.fit$par[(g+1):(2*g-1)]
                lambda.hat <- c((cumprod(alpha.out[(g-1):1]))[(g-1):1],1)*lam.g
                pi.hat     <- c(pi.out,1-sum(pi.out))
                temp.mat   <- matrix(NA,g,maxcol)
                for (iii in 1:g) for (jjj in 1:maxcol)
                    temp.mat[iii,jjj] <-
                        pi.hat[iii]*dpois(jjj-1,lambda.hat[iii])
                probs.mat[g,] <- apply(temp.mat,2,sum)
            }
        }
        pois.probs.mat[sp,] <- probs.mat[AIC.vect==min(AIC.vect),]
    }
    ## print(round(pois.probs.mat,3))
    ## Now construct the NO matrix:
    out.mat <- matrix(1,ns,ns)
    for (ii in 1:(ns-1)){
        for (jj in (ii+1):ns){
            small.mat <- pois.probs.mat[c(ii,jj),]
            mins <- apply(small.mat,2,min)
            out.mat[ii,jj] <- sum(mins)
            out.mat[jj,ii] <- sum(mins)
        }
    }
    out.mat
}     

## Function to find -loglikelihood, Poisson mixture model, g groups:
## Used for count data.

pois.minusll.g <- function(invect){
    alpha.in  <- invect[1:(g-1)]
    lam.v     <- rep(invect[g],g)
    pi.in     <- invect[(g+1):(2*g-1)]
    lam.v[-g] <- lam.v[g]*((cumprod(alpha.in[(g-1):1]))[(g-1):1])
    pi.v      <- c(pi.in,1-sum(pi.in))
    loglik    <- -10^8
    if (min(c(lam.v,pi.v,1-pi.v))>0){
        loglik  <- 0
        for (xi in x){
            term <- sum(pi.v*exp(-lam.v)*(lam.v^xi))
            loglik <- loglik + log.fn(term)
        }
    }
    -loglik 
}

## Niche overlap for categorical data:

no.cat.fn <- function(sp,yy){
    counts.mat <- as.matrix(table(sp,yy))
    props.mat  <- counts.mat/apply(counts.mat,1,sum)
    out.mat <- matrix(1,ns,ns)
    for (ii in 1:(ns-1)) for (jj in (ii+1):ns){
        small.mat <- props.mat[c(ii,jj),]
        mins <- apply(small.mat,2,min)
        out.mat[ii,jj] <- sum(mins)
        out.mat[jj,ii] <- sum(mins)
    }
    out.mat
}

## Niche overlap for continuous data:

no.cts.fn <- function(sp,yy){
    ## Do density estimation for each species:
    ## The density functions must have matching values of x.
    ## Go well outside the actual data at ends, to get to 
    ## zero density.
    lowx <- min(yy)-2*sd(yy)
    topx <- max(yy)+2*sd(yy)
    ## Define the x values for which the density functions
    ## will be calculated. These must match. Go well
    ## outside the actual data at top end, to get to zero density.
    ## Lowest value at 0, highest at topx.
    ## Let nn = number of x values.
    nn   <- 500
    ## Find dd, the distance between spikes:
    dd <- (topx-lowx)/(nn-1)
    x.vect <- seq(lowx,topx,length.out=nn)
    y.mat  <- matrix(NA,ns,nn)
    for (ii in 1:ns){
        this.sp <- snames[ii]
        this.y  <- yy[sp==this.sp]
        this.y  <- this.y[!is.na(this.y)]  ## Remove missing values
        this.n  <- length(this.y)          ## Number of usable observations
        ## Need at least two points to select a bandwidth for density est.
        if (length(this.y)>1){      ## Density estimation is possible
            this.dens <- density(this.y, n=nn, from=lowx, to=topx)
            y.mat[ii,] <- this.dens[[2]]
        }
    }
    ## Find niche overlap for each pair of species.
    out.mat  <- matrix(1,ns,ns)
    for (ii in 1:(ns-1)) for (jj in (ii+1):ns){
        min.dens <- apply(y.mat[c(ii,jj),],2,min)
        this.no  <- sum(min.dens)*dd
        out.mat[ii,jj] <- this.no 
        out.mat[jj,ii] <- this.no
    }
    out.mat
}



## ----------------------------------------------------------------
##                Calculate Manly's alpha
## ----------------------------------------------------------------

## Function to calculate the alpha values for different spp at one site.

alpha.fn <- function(sp,yy,avail.v){
    out.mat <- matrix(NA,ns,no.choices)
    for (ii in 1:ns){
        this.sp <- snames[ii]
        this.f.vect <- rep(0,no.choices)
        for (jj in 1:no.choices){
            this.choice <- choicenames[jj]
            this.f.vect[jj] <- 
                length(yy[(yy==this.choice)&(sp==this.sp)])
        }
        alpha.v <- rep(NA,no.choices)
        alpha.v[avail.v>0] <- 
            this.f.vect[avail.v>0]/avail.v[avail.v>0]
        if (sum(alpha.v,na.rm=T)>0)
            out.mat[ii,] <- alpha.v/sum(alpha.v,na.rm=T)
    }
    out.mat 
}

## Function to calculate the alpha values for one species,
## different sites.

alpha2.fn <- function(si,yy,avail.m){
    out.mat <- matrix(NA,ns,no.choices)
    for (ii in 1:ns){  ## Site ii
        this.si <- snames[ii]
        this.f.vect <- rep(0,no.choices)
        for (jj in 1:no.choices) {
            this.choice <- choicenames[jj]
            this.f.vect[jj] <- 
                length(yy[(yy==this.choice)&(si==this.si)])
        }
        alpha.v <- rep(NA,no.choices)
        avail.v <- avail.m[ii,]
        alpha.v[avail.v>0] <- 
            this.f.vect[avail.v>0]/avail.v[avail.v>0]
        if (sum(alpha.v,na.rm=T)>0)
            out.mat[ii,] <- alpha.v/sum(alpha.v,na.rm=T)
    }
    out.mat 
}



## ----------------------------------------------------------------
##       Calculate the NOs for resource selection variables
## ----------------------------------------------------------------

## Niche overlap for resource selection, either several species at 
## one site, or one pecies over several sites.
## Treats the alphas as proportions in different categories (choices). 
## Input is a matrix, rows = spp or sites, cols = choices. 
## Row sums are 1.

no.rsel.cat.fn <- function(alpmat){
    props.mat  <- alpmat
    out.mat <- matrix(1,ns,ns)
    for (ii in 1:(ns-1)) for (jj in (ii+1):ns){
        small.mat <- props.mat[c(ii,jj),]
        mins <- apply(small.mat,2,min)
        out.mat[ii,jj] <- sum(mins)
        out.mat[jj,ii] <- sum(mins)
    }
    out.mat
}
